diff -Nuar uboot/board/VSC452EVB/VSC452EVB.c uboot_new/board/VSC452EVB/VSC452EVB.c
--- uboot/board/VSC452EVB/VSC452EVB.c	1969-12-31 19:00:00.000000000 -0500
+++ uboot_new/board/VSC452EVB/VSC452EVB.c	2009-11-04 10:08:14.000000000 -0500
@@ -0,0 +1,323 @@
+/* 
+ * (C) Copyright 2003
+ * Thomas.Lange@corelatus.se
+ *
+ * See file CREDITS for list of people who contributed to this
+ * project.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of
+ * the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
+ * MA 02111-1307 USA
+ */
+
+#include <common.h>
+#include <command.h>
+#include <asm/mipsregs.h>
+#include <asm/io.h>
+#include <vsc452/vsc452regs.h>
+
+#define MAX_GPIO_PORTS          5
+#define MAX_GPIOS		80
+#define GPIO_PORT0		0x10200a00
+#define GPIO_OFFSET		4
+#define INPUT_INTR_DISABLE	0x00000003
+#define GPIO(port, number)	GPIO_PORT0 + (port *0x40) + (number * GPIO_OFFSET)
+#define PHY_RESET		GPIO(1,11)
+
+#define TIMER_4_INITIAL_COUNT   0x10200180
+#define TIMER_4_CURRENT_COUNT   0x10200184
+#define TIMER_4_MODE_CONTROL    0x10200190
+#define TIMER_4_INT_STATUS      0x10200194
+#define PARALLEL_CHIP_SELECT_1_CTRL	0x10200004
+
+void UbootWatchdogEnable(void);
+void UbootWatchdogDisable(void);
+
+#define write32(addr, val)	writel(val, 0xa0000000 + (addr))
+#define read32(addr)		readl(0xa0000000 + (addr))
+
+
+long int initdram(int board_type)
+{
+	/* Sdram is setup by assembler code */
+	/* If memory could be changed, we should return the true value here */
+	return CFG_SDRAM_LEN;
+}
+
+
+/* Initialise GPIO pins */
+unsigned int
+initialize_gpio (void)
+{
+	int i;
+	uint32_t GPIO_interrupt_status[MAX_GPIO_PORTS] = {0x1020080C, 0x1020081C, 0x1020082C, 0x1020083C, 0x1020084C}; /* These are register offset addresses */
+
+	for(i=0; i < MAX_GPIOS; i++)
+	{
+		write32((GPIO_PORT0+(i*GPIO_OFFSET)), INPUT_INTR_DISABLE);
+	}
+
+	/* Clear all pending GPIO interrupt */
+	for(i=0; i < MAX_GPIO_PORTS; i++)
+ 	{
+ 		write32(GPIO_interrupt_status[i], 0xFFFF);
+	}
+
+	return 0;
+}
+
+
+int checkboard (void)
+{
+	write32(PHY_RESET, 0); /* Toggle the reset line of the PHY */
+	udelay(10);
+	write32(PHY_RESET, 1);
+	
+       /* Enable the SP access of FPGA registers via Flash Chip select 1 (FSC1)  */
+	write32(PARALLEL_CHIP_SELECT_1_CTRL, 0x00000012);
+
+	return 0;
+}
+
+short 
+ReadJumperConfig(void)
+{
+	/* Return Path ID */	
+	return -1;	
+}
+
+/*
+ * Do watchdog interrupt disabling - its needed in case of cold reset
+ * We are supporting 3,4,7 and 8 Timers.
+ *
+ */
+unsigned int
+disable_watchdog (void)
+{
+       /* Timer 3 */
+       *((unsigned long *)0xB0200170) = 0;
+       *((unsigned long *)0xB0200174) = 0;
+
+       /* Timer 4 */
+       *((unsigned long *)0xB0200190) = 0;
+       *((unsigned long *)0xB0200194) = 0;
+
+       /* Timer 7 */
+       *((unsigned long *)0xB02001F0) = 0;
+       *((unsigned long *)0xB02001F4) = 0;
+
+       /* Timer 8 */
+       *((unsigned long *)0xB0200210) = 0;
+       *((unsigned long *)0xB0200214) = 0;
+
+       return 0;
+
+}
+
+/*
+ * Using timer 4 as Uboot Watchdog timer  and this timer  shall be reused again in Kernel, as SP Kernel Watchdog Timer with timer 3
+ *   
+ */
+
+void
+UbootWatchdogEnable(void)
+{
+        write32(TIMER_4_INITIAL_COUNT, 0x773593FF);
+        write32(TIMER_4_INT_STATUS, 0x01);
+        write32(TIMER_4_MODE_CONTROL, 0x03);
+
+        return;
+}
+
+void
+UbootWatchdogDisable(void)
+{
+        write32(TIMER_4_INITIAL_COUNT, 0x00);
+        write32(TIMER_4_MODE_CONTROL, 0x00);
+        write32(TIMER_4_INT_STATUS, 0x00);
+
+        return;
+}
+
+
+#ifdef CFG_READ_MAC_FROM_FRU_SUPPORT
+
+/* 
+ * Read Platform from FRU 
+ * 
+ * Set the platformid as Absolute if string  "Absolut" or "SB_XVIII" ( "PLAT:SB_XXXX;" ) found in FRU.  
+ * 
+ */
+int
+parsePlatformidAndSet (unsigned char *FruBuf, char * env_platformid)
+{
+	
+	int i ;
+	char ReferenceStr[10];
+	char BuellStr[6];
+	char BimotaStr[7];
+	char AgustaStr[7];
+	
+	char platStr[6];
+	char TempBuf[9]={0};
+	
+	memcpy(ReferenceStr, "Reference", 9);
+	memcpy(BuellStr, "BUELL", 5);
+	memcpy(BimotaStr, "BIMOTA", 6);
+	memcpy(AgustaStr, "AGUSTA", 6);
+
+	memcpy(platStr, "PLAT:", 5);
+	
+	for (i=0; i<256-6; i++)
+	{
+		/* Search platfrom id string "Buell"  */
+		if (0 == memcmp(FruBuf+i, BuellStr, 5))
+		{
+			strcpy (env_platformid, "Buell");	
+			break;
+		}
+		
+		/* Search platfrom id string "Bimota"  */
+		if (0 == memcmp(FruBuf+i, BimotaStr, 6))
+		{
+			strcpy(env_platformid, "Bimota");
+			break;
+		}
+
+		/* Search platfrom id string "Agusta"  */
+		if (0 == memcmp(FruBuf+i, AgustaStr, 6))
+		{
+			strcpy(env_platformid, "Agusta");	
+			break;
+		}
+
+		/* Search platfrom id string "Reference" */
+		if (0 == memcmp(FruBuf+i, ReferenceStr, 9))
+		{
+			strcpy(env_platformid, "Reference");
+			break;
+		} 
+
+		/* Search platfrom id string "PLAT:" */
+		if (0 == memcmp(FruBuf+i, platStr, 5))
+		{
+			/* 
+		        *If found, copy ASCII platrom "PLAT:SB_XXXX;"  . Store the value of "SB_XXXX"  in TempBuf array and append "\0"as  
+			*                              ^-- starts here 
+                       *      to last element of array to make it an string ( for string comparison ) 
+			*/
+			int j=0; 
+			/*
+			*	"PLAT:SB_XXXX" 
+			*             ^--j starts here [ *(FruBuf+i+5+j) ]
+			*
+			*/
+			for (j=0; j<=8; j++)  
+			{	
+				/*Store the values TempBuf in untill we found the ascii ";"  */
+				if (*(FruBuf+i+5+j) == ';') 
+				{
+					/* Append last element of array with '\0' to make it as string*/
+					TempBuf[j] = '\0'; 
+					break;
+				}		
+				else 
+				{
+					TempBuf[j] = *(FruBuf+i+5+j);
+				}
+			}
+			
+			if ((0 == strncmp(TempBuf, "SB_XX", 6)) || (0 == strncmp(TempBuf, "XX", 3)))
+			{
+				strcpy (env_platformid, "Buell");
+				break ;
+			}
+
+			if (0 == strncmp(TempBuf,"SB_XXI", 7))
+			{
+				strcpy (env_platformid , "Bimota");
+				break ;
+			}
+			if (0 == strncmp(TempBuf, "SB_XXII", 8))
+			{
+				strcpy (env_platformid, "Agusta");
+				break ;
+			}
+			
+		} 
+		
+	}
+	if (i >= 256-6)
+	{
+		printf("Did not find platfromid in FruBuf\n");
+		return -1;
+	}
+	return 0;
+
+}
+
+#endif 
+
+/*
+ * Read the platformid  from EEPROM.
+ * 
+ */
+unsigned int 
+readplatformid ( char *buf  )
+{
+	char platformid [256] = "Reference";
+	char *env_platformid = NULL;
+	
+#ifdef CFG_READ_MAC_FROM_FRU_SUPPORT
+
+	/* Get the platformid from FRU EEPROM */
+	int retval ;
+	extern int ReadFru (unsigned char *);
+	unsigned char FruBuf[256] = { 0 };
+	char platformStr [256] = { 0 };
+
+	/* Read FRU data */
+	retval = ReadFru (FruBuf);
+	if ( 0 == retval )
+	{
+		/* Obtain platformid from FRU data */
+		retval = parsePlatformidAndSet (FruBuf,platformStr);
+		if ( 0 == retval )
+		{
+			strcpy (buf,platformStr);	
+			return strlen(platformStr);
+	      	}	
+	}
+	else 
+	{
+		printf("WARNING: platformid could not be parsed from FRU, perhaps it is not programmed \n");
+	}
+
+#endif 	
+
+	/* Get  the platformid from uboot environment variable , if  EEPROM dont have the platformid */
+	env_platformid = getenv("platformid");
+	if ( NULL != env_platformid )
+	{
+	       if ((0 == strncmp(env_platformid, "Reference", 9) ) ||( 0 == strncmp(env_platformid, "Buell", 5) )|| (0 == strncmp(env_platformid, "Bimota", 6)) || ( 0== strncmp(env_platformid, "Agusta", 6))) 
+		{
+			strcpy(buf, env_platformid);
+			return strlen(env_platformid);
+		}	
+	}	
+
+	/* Set the default platform ID as Reference , if both EEPROM and uboot environment variable dont have the platformid  */
+	strcpy (buf, platformid);
+	return strlen(platformid);
+}
